<?php
/*
 * uses "nmap" to scan a host for the list of open ports, also to determine apps runing and OS type of the machine
 * uses "php" native functions to check port states
 * v00.01
 *
 * edit line-35 to adjust custom nmap parameters
 */

$host = @$_SERVER['argv'][1];
$cmd = @$_SERVER['argv'][2];
$task = @$_SERVER['argv'][3];
$port = @$_SERVER['argv'][4];

$sock_timeout=10;//seconds
$outputf="/tmp/$host.xml";
$cachef="/tmp/$host.dmp";

//uses cached information gathered by nmap
if ($cmd=='query') {
    if (!file_exists($cachef)) die("querie cache file not found\n");
    else include($cachef);
    $details=$all['details'];
    $services=$all['services'];
}

//first time run - nmap's used to gather info and save results in /tmp/ folder
if ($cmd=='index') {
        $descriptorspec = array(
           0 => array("pipe", "r"),	//stdin
           1 => array("pipe", "w"),	//stdout
           2 => array("pipe", "r") 	// stderr
        );
        
        $process = proc_open("/usr/local/bin/nmap -A -T4 -oX $outputf $host 2>&1", $descriptorspec, $pipes);
        
        if (is_resource($process)) {
            while(is_alive($process)) {
        	$resp=stream_get_contents($pipes[1]);//gran EOF and terminate loop
            }
            proc_close($process);
        }
        
        $services=array();
        $details=array(
            'os_vendor'=>'?',
            'os_family'=>'?',
            'os_gen'=>'?',
            'os_dets'=>'?',
            'os_accu'=>'?'
        );
        
        $r = new XMLReader();
        $r->open($outputf);
        while ($r->read()) {
            switch ($r->nodeType) {
        	case (XMLREADER::ELEMENT):
        	    switch ($r->localName) {
        		case "port":
        		    if (isset($port) && !isset($services[$port])) {
        			$services[$port]=array(
        			    'protocol'=>$proto,
        			    'port'=>$port,
        			    'port-state'=>$state,
        			    'service'=>$service,
        			    'app'=>$product,
        			    'app-ver'=>$version,
        			    'app-info'=>$info
        			);
        		    }
        		    $proto=$r->getAttribute("protocol");
        		    $port=$r->getAttribute("portid");
                    	    break;
                    	case "state":
                    	    $state=$r->getAttribute("state");
                    	    break;
                    	case "service":
                    	    $service=$r->getAttribute("name");
                    	    $product=$r->getAttribute("product");
                    	    $version=$r->getAttribute("version");
                    	    $info=$r->getAttribute("extrainfo");
                    	    break;
        		case "osclass":
        		    $os_vendor=$r->getAttribute("vendor");
        		    $os_family=$r->getAttribute("osfamily");
        		    $os_gen=$r->getAttribute("osgen");
        		    
        		    $details['os_vendor']=$os_vendor;
        		    $details['os_family']=$os_family;
        		    $details['os_gen']=$os_gen;
        		    break;
        		case "osmatch":
        		    $os_dets=$r->getAttribute("name");
        		    $os_accu=$r->getAttribute("accuracy");
        		    
        		    $details['os_dets']=$os_dets;
        		    $details['os_accu']=$os_accu;
        		    break;
        		case "finished":
        		    $duration=$r->getAttribute("elapsed");
        		    break;
                    	default:
                            break;
                    }
            }
        }
        
        if (isset($port) && !isset($services[$port])) {
        	$services[$port]=array(
        	    'protocol'=>$proto,
        	    'port'=>$port,
        	    'port-state'=>$state,
        	    'service'=>$service,
        	    'app'=>$product,
        	    'app-ver'=>$version,
        	    'app-info'=>$info
        	);
        }
	
        $details['duration']=$duration;
        
        //print_r($services);
        //print_r($details);
        
        $all=array('services'=>$services,'details'=>$details);//prepare for the cacheing
        
        //dump structure
        ob_start();
        var_export($all);
        $all=ob_get_contents();
        ob_end_clean();
        
        //save data to the file
        $fp = fopen($cachef, 'w');
        if (!$fp) die('can not cache data');
        $state=fwrite($fp, "<?php \$all=$all;\n?>");
        if(!$state) die('can not cache data');
        fclose($fp);
        
        foreach ($services as $port => $details) {
            echo $port."\n";
        }
} elseif ($cmd=='query' && $task=='index') {
    foreach ($services as $port => $d) echo "$port:\n";
} elseif ($cmd=='query' && $task=='port') {
    foreach ($services as $port => $d) echo "$port:$port\n";
} elseif ($cmd=='query' && $task=='proto') {
    foreach ($services as $port => $d) echo "$port:${d['protocol']}\n";
} elseif ($cmd=='query' && $task=='port-state') {
    foreach ($services as $port => $d) echo "$port:${d['port-state']}\n";
} elseif ($cmd=='query' && $task=='service') {
    foreach ($services as $port => $d) echo "$port:${d['service']}\n";
} elseif ($cmd=='query' && $task=='app') {
    foreach ($services as $port => $d) echo "$port:${d['app']}\n";
} elseif ($cmd=='query' && $task=='app-ver') {
    foreach ($services as $port => $d) echo "$port:${d['app-ver']}\n";
} elseif ($cmd=='query' && $task=='app-info') {
    foreach ($services as $port => $d) echo "$port:${d['app-info']}\n";
} elseif ($cmd=='query' && $task=='os-vendor') {
    foreach ($services as $port => $d) echo "$port:${details['os_vendor']}\n";
} elseif ($cmd=='query' && $task=='os-family') {
    foreach ($services as $port => $d) echo "$port:${details['os_family']}\n";
} elseif ($cmd=='query' && $task=='os-gen') {
    foreach ($services as $port => $d) echo "$port:${details['os_gen']}\n";
} elseif ($cmd=='query' && $task=='os-dets') {
    foreach ($services as $port => $d) echo "$port:${details['os_dets']}\n";
} elseif ($cmd=='query' && $task=='os-accu') {
    foreach ($services as $port => $d) echo "$port:${details['os_accu']}\n";
} elseif ($cmd=='query' && $task=='duration') {
    foreach ($services as $port => $d) echo "$port:${details['duration']}\n";
} elseif ($cmd=='query' && $task='statuses') {
    foreach ($services as $port => $d) echo "$port:".getPortStatus($host,$port,$sock_timeout)."\n";
} elseif ($cmd=='check' && $task=='port' && preg_match('/\d+/',$port,$m)) {
    echo getPortStatus($host,$port,$sock_timeout)."\n";
} else {
    die("usage #php ".$_SERVER['argv'][0]." [var1] [var2] [var3] [var4]\n");
}

/*
 * "udp://localhost" for checking udp sockets (not yet implemented)
 */
function getPortStatus($host,$port,$timeout) {
    $ts = microtime_float();
    $conn = @fsockopen($host,$port,$errno,$errstr,$timeout);
    if ($conn) {
	$delta=round((microtime_float()-$ts),6);
	return $delta;
        fclose($conn);
    } else {
	return "ERROR-$errno ($errstr)";
    }
}

function microtime_float() {
    list($usec, $sec) = explode(" ", microtime());
    return ((float)$usec + (float)$sec);
}

function is_alive($proc) {
    $dets=proc_get_status($proc);
    return $dets['running'];
}
?>